package edu.sdsc.inca;

import edu.sdsc.inca.protocol.Protocol;
import edu.sdsc.inca.util.ExpandablePattern;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;

/**
 * A Dialog window that allows the user to edit a resource.
 */
public class IncatResourceDialog extends JDialog implements ActionListener {

  protected JComboBox access;
  protected JPanel accessPanel;
  protected JPanel accessGlobusPanel;
  protected JPanel accessLocalPanel;
  protected JPanel accessManualPanel;
  protected JPanel accessSshPanel;
  protected JTextField adminEmail;
  protected JTextField computeServer;
  protected JTextField fileServer;
  protected JCheckBox isEquivalent;
  protected JTextField keyFile;
  protected JPasswordField keyPassword;
  protected ActionListener listener;
  protected JTextField members;
  protected JTextField name;
  protected JTextField proxyLifetime;
  protected JPasswordField proxyPassword;
  protected JTextField proxyServer;
  protected JTextField proxyUserId;
  protected JTextField userId;
  protected JTextField wd;

  /**
   * Constructs a new IncatResourceDialog.
   *
   * @param listener the listener to invoke when OK or cancel is pressed
   * @param okCommand the command to send when OK is pressed
   * @param cancelCommand the command to send when Cancel is pressed
   */
  public IncatResourceDialog(ActionListener listener,
                             String okCommand,
                             String cancelCommand) {
    this.adminEmail = IncatComponents.JTextFieldFactory(20, okCommand, this);
    this.computeServer = IncatComponents.JTextFieldFactory(20, okCommand, this);
    this.fileServer = IncatComponents.JTextFieldFactory(20, okCommand, this);
    this.isEquivalent = new JCheckBox("Equivalent");
    this.keyFile = IncatComponents.JTextFieldFactory(30, okCommand, this);
    this.keyPassword =
      IncatComponents.JPasswordFieldFactory(10, okCommand, this);
    this.members = IncatComponents.JTextFieldFactory(15, okCommand, this);
    this.name = IncatComponents.JTextFieldFactory(15, okCommand, this);
    this.proxyLifetime = IncatComponents.JTextFieldFactory(5, okCommand, this);
    this.proxyPassword =
      IncatComponents.JPasswordFieldFactory(10, okCommand, this);
    this.proxyServer = IncatComponents.JTextFieldFactory(20, okCommand, this);
    this.proxyUserId = IncatComponents.JTextFieldFactory(10, okCommand, this);
    this.userId = IncatComponents.JTextFieldFactory(10, okCommand, this);
    this.wd = IncatComponents.JTextFieldFactory(30, okCommand, this);
    this.access = new JComboBox(Protocol.COMPUTE_METHODS);
    this.access.addActionListener(this);
    this.access.setActionCommand("accesschange");
    this.accessPanel = new JPanel();
    this.accessGlobusPanel =
      IncatComponents.JPanelFactory(new JComponent[] {
        new JLabel("Gram Server "), this.computeServer, null,
        new JLabel("GridFtp Server "), this.fileServer, null
      }, false
    );
    this.accessLocalPanel =
      IncatComponents.JPanelFactory(new JComponent[] {
      }, false
    );
    this.accessManualPanel =
      IncatComponents.JPanelFactory(new JComponent[] {
        new JLabel("Resource Admin Email "), this.adminEmail
      }, false
    );
    this.accessSshPanel =
      IncatComponents.JPanelFactory(new JComponent[] {
        new JLabel("User Id "), this.userId, null,
        new JLabel("Key File "), this.keyFile, null,
        new JLabel("Password "), this.keyPassword, null
      }, false
    );
    this.setContentPane(IncatComponents.JPanelFactory(new JComponent[] {
      IncatComponents.JPanelFactory(new JComponent[] {
        this.name, null,
        new JLabel("Group Name *"), null
      }, false),
      IncatComponents.JPanelFactory(new JComponent[] {
        this.members, null,
        new JLabel("Members *"), null,
        this.isEquivalent, null
      }, false),
      IncatComponents.JPanelFactory(new JComponent[] {
        new JLabel("Access Method "), this.access, null,
        this.accessPanel, null
      }, false), null,
      IncatComponents.JPanelFactory(new JComponent[] {
        new JLabel("Install Directory "), this.wd, null,
        new JLabel("Proxy Server "), this.proxyServer, null,
        new JLabel("Proxy User ID "), this.proxyUserId, null,
        new JLabel("Proxy Password "), this.proxyPassword, null,
        new JLabel("Proxy Lifetime "), this.proxyLifetime, null,
      }, false), null,
      IncatComponents.JPanelFactory(new JComponent[] {
        IncatComponents.JButtonFactory("Cancel", cancelCommand, listener),
        IncatComponents.JButtonFactory("Ok", okCommand, this), null
      }, false), null
    }));
    this.setTitle("Incat Resource Dialog");
    this.pack();
    this.listener = listener;
  }

  /**
   * Returns the resource name.
   *
   * @return the resource name
   */
  public String getName() {
    return this.name.getText();
  }

  /**
   * Copies information from the resource dialog into the specified resource.
   *
   * @param resource the resource to update
   */
  public void getResource(WrapResource resource) {
    resource.setName(this.name.getText());
    resource.setMacroValue(Protocol.GROUPNAME_MACRO, this.name.getText());
    String pattern = this.members.getText();
    resource.setMacroValue(Protocol.PATTERN_MACRO, pattern);
    pattern =
      new ExpandablePattern(pattern.replaceAll("[ ,]", "|"), true).toString();
    resource.setXpath("//resource[matches(name, '^(" + pattern + ")$')]");
    String method = (String)this.access.getSelectedItem();
    resource.setMacroValue(Protocol.COMPUTE_METHOD_MACRO, method);
    // Clear all method-specific fields, then set the appropriate ones
    resource.setServer(WrapResource.COMPUTE_SERVER, "");
    resource.setServer(WrapResource.FILE_SERVER, "");
    resource.removeMacro(Protocol.EMAIL_MACRO);
    resource.removeMacro(Protocol.LOGIN_ID_MACRO);
    resource.removeMacro(Protocol.SSH_IDENTITY_MACRO);
    resource.removeMacro(Protocol.SSH_PASSWORD_MACRO);
    if(method.startsWith("globus")) {
      resource.setServer
        (WrapResource.COMPUTE_SERVER, this.computeServer.getText());
      resource.setServer(WrapResource.FILE_SERVER, this.fileServer.getText());
    } else if(method.equals("manual")) {
      resource.setMacroValue
        (Protocol.EMAIL_MACRO, this.adminEmail.getText(), "");
    } else if(method.equals("ssh")) {
      resource.setMacroValue
        (Protocol.LOGIN_ID_MACRO, this.userId.getText(), "");
      resource.setMacroValue
        (Protocol.SSH_IDENTITY_MACRO, this.keyFile.getText(), "");
      resource.setMacroValue
        (Protocol.SSH_PASSWORD_MACRO,
         new String(this.keyPassword.getPassword()), "");
    }
    resource.setServer(WrapResource.PROXY_SERVER, this.proxyServer.getText());
    resource.setMacroValue
      (Protocol.MYPROXY_LIFETIME_MACRO, this.proxyLifetime.getText(), "");
    resource.setMacroValue(Protocol.MYPROXY_PASSWORD_MACRO,
                           new String(this.proxyPassword.getPassword()), "");
    resource.setMacroValue
      (Protocol.MYPROXY_USERNAME_MACRO, this.proxyUserId.getText(), "");
    resource.setMacroValue
      (Protocol.EQUIVALENT_MACRO,
       new Boolean(this.isEquivalent.isSelected()).toString());
    resource.setMacroValue(Protocol.WORKING_DIR_MACRO, this.wd.getText(), "");
  }

  /**
   * Copies information from the specified resource into the resource dialog.
   *
   * @param resource the resource to show
   */
  public void setResource(WrapResource resource) {
    this.name.setText(resource.getName());
    this.members.setText(resource.getMacroValue(Protocol.PATTERN_MACRO));
    String method = resource.getMacroValue(Protocol.COMPUTE_METHOD_MACRO);
    IncatComponents.setJComboBoxSelectedString
      (this.access, method == null ? Protocol.COMPUTE_METHODS[0] : method);
    this.adminEmail.setText(resource.getMacroValue(Protocol.EMAIL_MACRO));
    this.keyFile.setText
      (resource.getMacroValue(Protocol.SSH_IDENTITY_MACRO));
    this.keyPassword.setText
      (resource.getMacroValue(Protocol.SSH_PASSWORD_MACRO));
    this.userId.setText(resource.getMacroValue(Protocol.LOGIN_ID_MACRO));
    this.computeServer.setText(resource.getServer(WrapResource.COMPUTE_SERVER));
    this.fileServer.setText(resource.getServer(WrapResource.FILE_SERVER));
    this.proxyServer.setText(resource.getServer(WrapResource.PROXY_SERVER));
    this.proxyLifetime.setText
      (resource.getMacroValue(Protocol.MYPROXY_LIFETIME_MACRO));
    this.proxyPassword.setText
      (resource.getMacroValue(Protocol.MYPROXY_PASSWORD_MACRO));
    this.proxyUserId.setText
      (resource.getMacroValue(Protocol.MYPROXY_USERNAME_MACRO));
    String state = resource.getMacroValue(Protocol.EQUIVALENT_MACRO);
    this.isEquivalent.setSelected(state != null && state.equals("true"));
    this.wd.setText(resource.getMacroValue(Protocol.WORKING_DIR_MACRO));
  }

  /**
   * An action listener that checks for required fields on dialog exit and
   * modifies the dialog when the access method changes.
   */
  public void actionPerformed(ActionEvent ae) {
    if(ae.getActionCommand().equals("accesschange")) {
      String method = (String)this.access.getSelectedItem();
      this.accessPanel.removeAll();
      if(method.startsWith("globus")) {
        this.accessPanel.add(this.accessGlobusPanel);
      } else if(method.equals("local")) {
        this.accessPanel.add(this.accessLocalPanel);
      } else if(method.equals("manual")) {
        this.accessPanel.add(this.accessManualPanel);
      } else if(method.equals("ssh")) {
        this.accessPanel.add(this.accessSshPanel);
      }
      this.pack();
    } else {
      if(this.name.getText().equals("") ||
         this.members.getText().equals("")) {
        JOptionPane.showMessageDialog
          (this, "Please provide a resource name and member set",
           "Incomplete", JOptionPane.ERROR_MESSAGE);
          return;
      }
      this.listener.actionPerformed(ae);
    }
  }

}
